function DefineWire(%char, %rot, %clearHook, %updateHook, %plantHook, %events, %name, %desc, %category, %color)
{
	if(%color $= "")
		%colID = GetWireColor("Black");
	else %colID = GetWireColor(%color);
	if($WireCatIndex[%category] $= "")
		DefineWireCategory(%category, %category);
	%catPos = $WireCatIndex[%category];
	//If you want a proper description call DefineWireCategory before defining your wires
	%pos = -1+$WireCatWires[%catPos]++;
	$WireChar[%catPos, %pos] = %char;
	$WireRot[%catPos, %pos] = %rot;
	$WireClearHook[%catPos, %pos] = %clearHook;
	$WireUpdateHook[%catPos, %pos] = %updateHook;
	$WirePlantHook[%catPos, %pos] = %plantHook;
	$WireEvents[%catPos, %pos] = %events;
	$WireName[%catPos, %pos] = %name;
	$WireDescription[%catPos, %pos] = %desc;
	$WireColorID[%catPos, %pos] = %colID;
}

function DefineWireCategory(%name, %desc)
{
	%cat = -1+$WireCats++;
	$WireCatWires[%cat] = 0;
	$WireCatName[%cat] = %name;
	$WireCatDesc[%cat] = %desc;
	$WireCatIndex[%name] = %cat;
}

package WiresPackage
{
	function Armor::onTrigger(%data, %this, %trig, %tog)
	{
		Parent::onTrigger(%data, %this, %trig, %tog);
		if(%trig != 0 || !%tog || !isObject(%cl = %this.client)) return;
		if(isObject(%img = %this.getMountedImage(0)) && %img.getID() == WirePlacerImage.getID())
		{
			%eye = %this.getEyePoint();
			%end = vectorAdd(%eye, vectorScale(%this.getEyeVector(), 8));
			%mask = $Typemasks::fxBrickObjectType | $Typemasks::TerrainObjectType;
			%ray = containerRaycast(%eye, %end, %mask);
			if(!isObject(%obj = firstWord(%ray))) return;
			%hitPos = getWords(%ray, 1, 3); %normal = getWords(%ray, 4, 6);
			if(vectorDist(%normal, "0 0 1") <= 0.005) %tempPos = vectorAdd(%hitPos, vectorScale(%normal, 0.1));
			else if(vectorDist(%normal, "0 0 -1") <= 0.005) %tempPos = vectorAdd(%hitPos, vectorScale(%normal, 0.1));
			else %tempPos = vectorAdd(%hitPos, vectorScale(%normal, 0.25));
			%xPos = mFloatLength((getWord(%tempPos, 0) - 0.25) * 2, 0) / 2 + 0.25;
			%yPos = mFloatLength((getWord(%tempPos, 1) - 0.25) * 2, 0) / 2 + 0.25;
			%zPos = mFloatLength((getWord(%tempPos, 2) - 0.1) * 5, 0) / 5 + 0.1;
			%realPos = %xPos SPC %yPos SPC %zPos;
			%ang = mAtan(getWord(%for = %this.getForwardVector(), 0), getWord(%for, 1));
			%ang = mFloatLength(%ang / ($PI / 2), 0) + 0; %ang += $WireRot[%cl.wireCat, %cl.wireID];
			while(%ang < 0) %ang += 4; while(%ang >= 4) %ang -= 4;
			%tempTrans = %realPos@" 0 0 1 "@(%ang*($PI/2));
			if(%cl.wireID $= "") %cl.wireID = 0;
			if(%cl.wireCat $= "") %cl.wireCat = 0;
			if(isObject(%temp = %this.tempbrick) && !%temp.wireBrick)
				%temp.delete();
			if(!isObject(%temp))
			{
				%this.tempbrick = new fxDtsBrick()
				{
					colorID = $WireColorID[%cl.wireCat, %cl.wireID];
					datablock = brick1x1FPrintData;
				};
				%temp = %this.tempbrick; %temp.wireBrick = 1;
				%temp.setChar($WireChar[%temp.wireCat = %cl.wireCat, %temp.wireID = %cl.wireID]);
				%cl.bottomPrint("\c2"@$WireName[%temp.wireID]@"<br>\c3"@$WireDescription[%temp.wireID], -1, 1);
			}
			%temp.setTransform(%tempTrans);
			%hook = $WireUpdateHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
	}
	
	function fxDtsBrick::onRemove(%brick)
	{
		Parent::onRemove(%brick);
		if(!%brick.isPlanted() && %brick.wireBrick)
		{
			%hook = $WireClearHook[%brick.wireCat, %brick.wireID];
			if(isFunction(%hook)) call(%hook, %brick);
		}
	}
	
	function fxDtsBrick::SetColor(%brick, %colID)
	{
		if(%brick.isPlanted() || !%brick.wireBrick)
			Parent::setColor(%brick, %colID);
	}
	
	function servercmdLight(%cl)
	{
		if(isObject(%pl = %cl.player) && isObject(%pl.getObjectMount()))
			Parent::servercmdPrevSeat(%cl);
		else if(isObject(%pl) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%hook = $WireClearHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
			%cl.wireCat++; if(%cl.wireCat >= $WireCats) %cl.wireCat = 0; %cl.wireID = 0;
			%newRot = %temp.getAngleID() - $WireRot[%temp.wireCat, %temp.wireID] + $WireRot[%cl.wireCat, %cl.wireID];
			while(%newRot < 0) %newRot += 4; while(%newRot >= 4) %newRot -= 4;
			%temp.setChar($WireChar[%temp.wireCat = %cl.wireCat, %temp.wireID = %cl.wireID]);
			%temp.wireBrick = 0; %temp.setColor($WireColorID[%temp.wireCat, %temp.wireID]); %temp.wireBrick = 1;
			%temp.setTransform(%temp.getPosition()@" 0 0 1 "@(%newRot*($PI/2)));
			%cl.centerPrint("<br><br><br>\c2"@$WireCatName[%temp.wireCat]@"<br>\c3"@$WireCatDesc[%temp.wireCat], 5);
			%cl.bottomPrint("\c2"@$WireName[%temp.wireCat, %temp.wireID]@"<br>\c3"@$WireDescription[%temp.wireCat, %temp.wireID], -1, 1);
			%hook = $WireUpdateHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
		else Parent::servercmdLight(%cl);
	}
	
	function servercmdPrevSeat(%cl)
	{
		if(isObject(%pl = %cl.player) && isObject(%pl.getObjectMount()))
			Parent::servercmdPrevSeat(%cl);
		else if(isObject(%pl) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%hook = $WireClearHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
			%cl.wireID++; if(%cl.wireID >= $WireCatWires[%cl.wireCat]) %cl.wireID = 0;
			%newRot = %temp.getAngleID() - $WireRot[%temp.wireCat, %temp.wireID] + $WireRot[%cl.wireCat, %cl.wireID];
			while(%newRot < 0) %newRot += 4; while(%newRot >= 4) %newRot -= 4;
			%temp.setChar($WireChar[%temp.wireCat = %cl.wireCat, %temp.wireID = %cl.wireID]);
			%temp.wireBrick = 0; %temp.setColor($WireColorID[%temp.wireCat, %temp.wireID]); %temp.wireBrick = 1;
			%temp.setTransform(%temp.getPosition()@" 0 0 1 "@(%newRot*($PI/2)));
			%cl.bottomPrint("\c2"@$WireName[%temp.wireCat, %temp.wireID]@"<br>\c3"@$WireDescription[%temp.wireCat, %temp.wireID], -1, 1);
			%hook = $WireUpdateHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
		else Parent::servercmdPrevSeat(%cl);
	}
	
	function servercmdNextSeat(%cl)
	{
		if(isObject(%pl = %cl.player) && isObject(%pl.getObjectMount()))
			Parent::servercmdNextSeat(%cl);
		else if(isObject(%pl) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%hook = $WireClearHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
			%cl.wireID--; if(%cl.wireID < 0) %cl.wireID = $WireCatWires[%cl.wireCat] - 1;
			%newRot = %temp.getAngleID() - $WireRot[%temp.wireCat, %temp.wireID] + $WireRot[%cl.wireCat, %cl.wireID];
			while(%newRot < 0) %newRot += 4; while(%newRot >= 4) %newRot -= 4;
			%temp.setChar($WireChar[%temp.wireCat = %cl.wireCat, %temp.wireID = %cl.wireID]);
			%temp.wireBrick = 0; %temp.setColor($WireColorID[%temp.wireCat, %temp.wireID]); %temp.wireBrick = 1;
			%temp.setTransform(%temp.getPosition()@" 0 0 1 "@(%newRot*($PI/2)));
			%cl.bottomPrint("\c2"@$WireName[%temp.wireCat, %temp.wireID]@"<br>\c3"@$WireDescription[%temp.wireCat, %temp.wireID], -1, 1);
			%hook = $WireUpdateHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
		else Parent::servercmdNextSeat(%cl);
	}
	
	function servercmdInstantUseBrick(%cl, %data)
	{
		if(isObject(%pl = %cl.player) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick) return;
		Parent::servercmdInstantUseBrick(%cl, %data);
	}
	
	function servercmdUseInventory(%cl, %slot)
	{
		if(isObject(%pl = %cl.player) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick) return;
		Parent::servercmdUseInventory(%cl, %slot);
	}
	
	function servercmdUseTool(%cl, %slot)
	{
		Parent::servercmdUseTool(%cl, %slot);
		if(isObject(%pl = %cl.player) && isObject(%img = %pl.getMountedImage(0)) && %img.getName() $= "WirePlacerImage")
		{
			if(%cl.hasntUsedWires)
				%cl.centerPrint("<br><br><br>\c2Use your \c3vehicle seat swap\c2 keys to change wires.<br>\c2To change categories, use your \c3light key\c2.", 5);
		}
	}
	
	function servercmdCancelBrick(%cl)
	{
		if(isObject(%pl = %cl.player) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%hook = $WireClearHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
		Parent::servercmdCancelBrick(%cl);
	}
	
	function servercmdShiftBrick(%cl, %for, %left, %up)
	{
		Parent::servercmdShiftBrick(%cl, %for, %left, %up);
		if(isObject(%pl = %cl.player) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%hook = $WireUpdateHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
	}
	
	function servercmdSuperShiftBrick(%cl, %for, %left, %up)
	{
		Parent::servercmdSuperShiftBrick(%cl, %for, %left, %up);
		if(isObject(%pl = %cl.player) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%hook = $WireUpdateHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
	}
	
	function servercmdRotateBrick(%cl, %rot)
	{
		Parent::servercmdRotateBrick(%cl, %rot);
		if(isObject(%pl = %cl.player) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%hook = $WireUpdateHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp);
		}
	}
	
	function servercmdPlantBrick(%cl)
	{
		%brick = Parent::servercmdPlantBrick(%cl);
		if(isObject(%brick) && isObject(%pl = %cl.player) && isObject(%temp = %pl.tempbrick) && %temp.wireBrick)
		{
			%events = $WireEvents[%temp.wireCat, %temp.wireID];
			for(%i=0;%i<getLineCount(%events);%i++)
			{
				%event = getLine(%events, %i);
				%enabled = getField(%event, 0);
				%delay = getField(%event, 1);
				%input = getField(%event, 2);
				%target = getField(%event, 3);
				%output = getField(%event, 4);
				for(%j=0;%j<getFieldCount(%event)-5;%j++)
					%arg[%j] = getField(%event, %j + 5);
				%len = strLen(%output);
				if(%len >= 5 && getSubStr(%output, %len - 5, 5) $= "North")
				{ %eventDir = 2; %mainEvent = getSubStr(%output, 0, %len - 5); }
				if(%len >= 4 && getSubStr(%output, %len - 4, 4) $= "East")
				{ %eventDir = 3; %mainEvent = getSubStr(%output, 0, %len - 4); }
				if(%len >= 5 && getSubStr(%output, %len - 5, 5) $= "South")
				{ %eventDir = 0; %mainEvent = getSubStr(%output, 0, %len - 5); }
				if(%len >= 4 && getSubStr(%output, %len - 4, 4) $= "West")
				{ %eventDir = 1; %mainEvent = getSubStr(%output, 0, %len - 4); }
				if(getSubStr(%output, 0, 9) $= "FireRelay")
				{
					%dir = getSubStr(%output, 9, strLen(%output) - 9);
					switch$(%dir)
					{
						case "North": %relayDir = 2;
						case "East": %relayDir = 3;
						case "South": %relayDir = 0;
						case "West": %relayDir = 1;
						default: %relayDir = -1;
					}
					if(%relayDir != -1)
					{
						%relayDir += %brick.angleID;
						while(%relayDir >= 4) %relayDir -= 4;
						switch(%relayDir)
						{
							case 0: %output = "FireRelaySouth";
							case 1: %output = "FireRelayWest";
							case 2: %output = "FireRelayNorth";
							case 3: %output = "FireRelayEast";
						}
					}
				}
				else if(%output $= "SetColor")
					%arg0 = GetWireColor(%arg0);
				else if(%output $= "SpawnProjectile")
				{
					switch(%brick.angleID)
					{
						case 0: %arg0Dup = %arg0;
						case 1: %arg0Dup = getWord(%arg0, 1) SPC -getWord(%arg0, 0) SPC getWord(%arg0, 2);
						case 2: %arg0Dup = -getWord(%arg0, 0) SPC -getWord(%arg0, 1) SPC getWord(%arg0, 2);
						case 3: %arg0Dup = -getWord(%arg0, 1) SPC getWord(%arg0, 0) SPC getWord(%arg0, 2);
					}
					%arg0 = %arg0Dup;
				}
				%brick.addEvent(%enabled, %delay, %input, %target, %output, %arg0, %arg1, %arg2, %arg3, %arg4);
			}
			%hook = $WirePlantHook[%temp.wireCat, %temp.wireID];
			if(isFunction(%hook)) call(%hook, %temp, %brick);
		}
		return %brick;
	}
};
activatePackage("WiresPackage");

datablock itemData(WirePlacerItem : PrintGun)
{
	colorShiftColor = "0 0 0 1";
	uiName = "Wire Placer";
	image = WirePlacerImage;
};

datablock shapeBaseImageData(WirePlacerImage : PrintGunImage)
{
	colorShiftColor = "0 0 0 1";
	item = WirePlacerItem;
};

function WirePlacerImage::onFire(%this, %obj, %slot) { return; }